// controls

import * as Inputs from "npm:@observablehq/inputs";
import random from "npm:random";
import shuffle from "npm:lodash.shuffle";
import * as d3 from "npm:d3-color";

const PALETTES = new Map([
	[ "random RGB", palette_random ], 
	[ "grayscale", palette_grayscale ],
    [ "monochrome", palette_monochrome ],
    [ "one spot", palette_one_spot ],
    [ "triad", triad_saturated ],
    [ "triad pastel", triad_pastel ],
    [ "triad dusk", triad_dusk ],
  	[ "RGB", palette_rgb ],
  	[ "RBY", palette_rby ],
  	[ "CMY", palette_cmy ],
]);

	// two spot
 	// RGB
 	// CMY
 	// RYB
 	// triad - full saturation
 	// triad - pastel
 	// trial - dusk
 	// random HSV
 	// random RGB

function palette_random() {
	const u = random.uniform(0, 255);
	return [1,2,3].map((x)=> d3.rgb(u(), u(), u()));
}


function palette_grayscale() {
	const u = random.uniform(0, 1);
	return [1,2,3].map((x)=> d3.hsl(0, 0, u()));
}

function palette_monochrome() {
	const u = random.uniform(0, 1);
	const h = u() * 360;
	return [1,2,3].map((x)=> d3.hsl(h, u(), u()));
}

function palette_one_spot() {
	const hue = random.uniform(0, 360);
	const cols = [ d3.color("white"), d3.color("black"), d3.hsl(hue(), 1, 0.5) ]
	return shuffle(cols);
}

function triad_saturated() {
	return triad(1, 0.5);
}

function triad_pastel() {
	return triad(0.6, 0.7);
}

function triad_dusk() {
	return triad(1, 0.25);
}

function triad(s, l) {
	const u = random.uniform(0, 360);
	const h1 = u();
	const h2 = (h1 + 120) % 360;
	const h3 = (h1 + 240) % 360;
	const cols = [ h1, h2, h3 ].map((h) => d3.hsl(h, s, l));
	return shuffle(cols); 
}

function palette_rgb() {
	const cols = [ d3.rgb(255, 0, 0), d3.rgb(0, 255, 0), d3.rgb(0, 0, 255) ];
	return shuffle(cols);
}

function palette_rby() {
	const cols = [ d3.rgb(255, 0, 0), d3.rgb(255, 255, 0), d3.rgb(0, 0, 255) ];
	return shuffle(cols);
}

function palette_cmy() {
	const cols = [ d3.rgb(0, 255, 255), d3.rgb(255, 255, 0), d3.rgb(255, 0, 255) ];
	return shuffle(cols);
}


class DotControls {

	constructor(fg, radius_opts) {	
		this.radius_opts = radius_opts;
		this.fg = Inputs.color({label: "colour", value: fg});
		this.m = Inputs.range([1, 5], {value: 2, step: 1, label:"m"});
		this.n = Inputs.range([1, 5], {value: 2, step: 1, label:"n"});
		this.r = Inputs.range([0, 0.4], {value: 0.2, step: 0.01, label: "radius"});
		this.f = Inputs.select(radius_opts, {label: "gradient", value: radius_opts[0]});
	}

	random_grid() {
		this.m.value = random.choice([1, 2, 3, 4, 5]);	
		this.n.value = random.choice([1, 2, 3, 4, 5]);
		this.r.value = random.float(0, 0.4);
		this.f.value = random.choice(this.radius_opts);
		this.m.dispatchEvent(new Event("input"));
		this.n.dispatchEvent(new Event("input"));
		this.r.dispatchEvent(new Event("input"));
		this.f.dispatchEvent(new Event("input"));
	}

	random_colours() {
		this.set_colour(random_colour());
	}

	set_colour(c) {
		this.fg.value = c;
		this.fg.dispatchEvent(new Event("input"));
	}


}

export { DotControls, PALETTES };